SystemOrganization addCategory: #QuantumLogic!
SystemOrganization addCategory: #'QuantumLogic-Engine'!
SystemOrganization addCategory: #'QuantumLogic-Example'!
SystemOrganization addCategory: #'QuantumLogic-Tests'!

Error subclass: #OutOfQuantumPossibility
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'QuantumLogic'!

Object subclass: #DepthFirstQuantumEngine
	instanceVariableNames: 'answers stack'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'QuantumLogic-Engine'!

!DepthFirstQuantumEngine methodsFor: 'as yet unclassified' stamp: 'hjo 3/27/2008 11:02'!
addToAnswers: anObject

	self answers add: anObject! !

!DepthFirstQuantumEngine methodsFor: 'accessing' stamp: 'hjo 2/2/2008 23:13'!
answers

	answers isNil ifTrue: [answers := OrderedCollection new].
	^answers! !

!DepthFirstQuantumEngine methodsFor: 'looping' stamp: 'hjo 2/2/2008 23:15'!
run

	[self stack isEmpty]
		whileFalse: [ self step ]! !

!DepthFirstQuantumEngine methodsFor: 'looping' stamp: 'hjo 2/2/2008 21:48'!
runToFindOneAnswer

	[ self answers isEmpty ] whileTrue: [ self step ]! !

!DepthFirstQuantumEngine methodsFor: 'accessing' stamp: 'hjo 2/2/2008 23:12'!
stack

	stack isNil ifTrue: [stack := Stack new].
	^stack! !

!DepthFirstQuantumEngine methodsFor: 'accessing' stamp: 'hjo 1/28/2012 19:38'!
stackContents

	^String streamContents: [:stream| 
		self stack do: [:sl| sl printOn: stream. stream cr ]
	]! !

!DepthFirstQuantumEngine methodsFor: 'testing' stamp: 'hjo 2/2/2008 23:15'!
stackIsEmpty

	^self stack isEmpty! !

!DepthFirstQuantumEngine methodsFor: 'as yet unclassified' stamp: 'hjo 1/29/2012 17:47'!
stackPopPossibilities

	^self stack pop possibilities! !

!DepthFirstQuantumEngine methodsFor: 'accessing' stamp: 'hjo 2/2/2008 23:14'!
stackSize
	
	^self stack size! !

!DepthFirstQuantumEngine methodsFor: 'accessing' stamp: 'hjo 1/29/2012 17:50'!
stackTop

	^self stack first! !

!DepthFirstQuantumEngine methodsFor: 'looping' stamp: 'hjo 1/29/2012 17:47'!
step

	"override this for more control over processing agents"
	
	self stackPopPossibilities do: 
		[:agent| 
			agent isDetermined
				ifTrue: [self addToAnswers: agent]
				ifFalse: [self stack push: agent]
		]! !

DepthFirstQuantumEngine subclass: #MaxFindingDepthFirstQuantumEngine
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'QuantumLogic-Engine'!

!MaxFindingDepthFirstQuantumEngine methodsFor: 'as yet unclassified' stamp: 'hjo 3/27/2008 10:48'!
addToAnswers: anEcologicalBinPacking 

	self answers put: anEcologicalBinPacking! !

!MaxFindingDepthFirstQuantumEngine methodsFor: 'as yet unclassified' stamp: 'hjo 3/27/2008 10:17'!
answers

	answers isNil ifTrue: [ answers := MaxClub new ].
	^answers! !

!MaxFindingDepthFirstQuantumEngine methodsFor: 'as yet unclassified' stamp: 'hjo 3/27/2008 10:17'!
evaluator: aBlockContext

	self answers evaluator: aBlockContext! !

!MaxFindingDepthFirstQuantumEngine methodsFor: 'as yet unclassified' stamp: 'hjo 1/29/2012 17:47'!
stackPopPossibilities

	| spreaded |
	spreaded := super stackPopPossibilities.
	^self answers maximum isNil
		ifTrue: [spreaded]
		ifFalse: [spreaded select: [:each| self answers mayAccept: each ]]! !

Object subclass: #QuantumObject
	instanceVariableNames: 'possibilities'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'QuantumLogic'!

!QuantumObject classSide methodsFor: 'instance creation' stamp: 'hjo 1/28/2012 17:54'!
possibilities: anArray 

	^self new possibilities: anArray  ! !

!QuantumObject methodsFor: 'changing' stamp: 'hjo 2/1/2012 14:06'!
be: anObject 

	self assert: [possibilities includes: anObject].  
	possibilities := Array with: anObject! !

!QuantumObject methodsFor: 'testing' stamp: 'hjo 1/28/2012 17:55'!
isDetermined

	^possibilities size = 1! !

!QuantumObject methodsFor: 'accessing' stamp: 'hjo 1/28/2012 19:47'!
possibilities

	^ possibilities! !

!QuantumObject methodsFor: 'accessing' stamp: 'hjo 1/28/2012 20:42'!
possibilities: anArray 

	self assert: [anArray notEmpty].
	possibilities := anArray ! !

!QuantumObject methodsFor: 'printing' stamp: 'hjo 1/28/2012 20:25'!
printOn: aStream

	self isDetermined 
		ifTrue: [aStream print: possibilities anyOne ]
		ifFalse: [super printOn: aStream ]! !

!QuantumObject methodsFor: 'changing' stamp: 'hjo 1/28/2012 20:18'!
removePossibility: anObject

	possibilities := possibilities copyWithout: anObject.
	possibilities isEmpty ifTrue: [OutOfQuantumPossibility new signal]! !

Object subclass: #QuantumSudoku
	instanceVariableNames: 'matrix'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'QuantumLogic-Example'!

!QuantumSudoku classSide methodsFor: 'as yet unclassified' stamp: 'hjo 1/28/2012 19:35'!
matrix: aMatrix

	^self new matrix: aMatrix! !

!QuantumSudoku methodsFor: 'quantumlogic' stamp: 'hjo 1/29/2012 17:31'!
isDetermined

	^matrix allSatisfy: [:each| each isDetermined ]  ! !

!QuantumSudoku methodsFor: 'accessing' stamp: 'hjo 1/28/2012 19:35'!
matrix
	^matrix! !

!QuantumSudoku methodsFor: 'accessing' stamp: 'hjo 1/28/2012 19:35'!
matrix: aMatrix

	matrix := aMatrix! !

!QuantumSudoku methodsFor: 'accessing' stamp: 'hjo 1/28/2012 19:45'!
matrixIndexOfAmbiguousElement

	self matrix withIndicesDo: [:x :r :c| 
		x isDetermined ifFalse: [^{r. c}] ].
	^nil! !

!QuantumSudoku methodsFor: 'quantumlogic' stamp: 'hjo 1/29/2012 17:30'!
possibilities

	|ambiguousIndex|
	ambiguousIndex := self matrixIndexOfAmbiguousElement.
	^ambiguousIndex isNil 
		ifTrue: [ {} ]
		ifFalse: [
			((matrix at: ambiguousIndex first at: ambiguousIndex second) possibilities collect: [:p| |cop|
				cop := self deepCopy.
				[cop putNumber: p at: ambiguousIndex first at: ambiguousIndex second] on: OutOfQuantumPossibility do: [:exp| 
					cop := nil.
				].
				cop
				]) reject: [:each| each isNil].
		]! !

!QuantumSudoku methodsFor: 'printing' stamp: 'hjo 1/28/2012 20:23'!
printOn: aStream

	super printOn:   aStream.
	aStream nextPut: $[.
	aStream print: matrix.
	aStream nextPut: $].  ! !

!QuantumSudoku methodsFor: 'changing' stamp: 'hjo 1/28/2012 20:15'!
putNumber: aNumber at: rowIndex at: columnIndex

	| determinedQuantum blockR blockC cellsAffected|
	determinedQuantum := matrix at: rowIndex at: columnIndex.
	determinedQuantum be: aNumber.
	
	cellsAffected := ((matrix atRow: rowIndex), (matrix atColumn: columnIndex), 
		[blockR := ((rowIndex - 1)//3)*3+1.
		blockC := ((columnIndex -1)//3)*3+1.
		(matrix atRows: blockR to: blockR +2 columns: blockC to:  blockC + 2) asArray] value) copyWithout: determinedQuantum.
	cellsAffected do: [:each|
		each removePossibility: aNumber  
		]! !

TestCase subclass: #QuantumObjectTest
	instanceVariableNames: 'only three'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'QuantumLogic-Tests'!

!QuantumObjectTest methodsFor: 'as yet unclassified' stamp: 'hjo 2/1/2012 14:01'!
setUp

	super setUp.
	three := QuantumObject possibilities: {1. 2. 3}.
	only := QuantumObject possibilities: {4}.! !

!QuantumObjectTest methodsFor: 'as yet unclassified' stamp: 'hjo 2/1/2012 14:01'!
testBe

	self assert: 3 equals: three possibilities size.
	three be: 2.
	self assert: 1 equals: three possibilities size.
	self flag: 'error when object is not in possibility'.! !

!QuantumObjectTest methodsFor: 'as yet unclassified' stamp: 'hjo 2/1/2012 14:05'!
testBeNotInThePossibilities

	self should: [three be: 200] raise: AssertionFailure.! !

!QuantumObjectTest methodsFor: 'as yet unclassified' stamp: 'hjo 2/1/2012 14:01'!
testIsDetermined

	self deny: three isDetermined.
	self assert: only isDetermined.! !

!QuantumObjectTest methodsFor: 'as yet unclassified' stamp: 'hjo 2/1/2012 13:54'!
testPossibilities

	self should: [QuantumObject possibilities: {}  ] raise: AssertionFailure ! !

!QuantumObjectTest methodsFor: 'as yet unclassified' stamp: 'hjo 2/1/2012 14:01'!
testRemovePossibility

	three removePossibility: 3.  
	self assert: 2 equals: three possibilities size.
	self should:  [only removePossibility: 4] raise: OutOfQuantumPossibility.
	  ! !
